﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;

namespace UniversityDatabaseImplement.Implements
{
    internal class ActivityStorage : IActivityStorage
    {
        public List<ActivityViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.Activities.Select(x => x.GetViewModel).ToList();
        }
        public List<ActivityViewModel> GetFilteredList(ActivitySearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.Activities
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id)
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ActivityViewModel? GetElement(ActivitySearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.Activities
            .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public ActivityViewModel? Insert(ActivityBindingModel model)
        {
            var newActivity = Activity.Create(model);
            if (newActivity == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.Activities.Add(newActivity);
            context.SaveChanges();
            return newActivity.GetViewModel;
        }

        public ActivityViewModel? Update(ActivityBindingModel model)
        {
            using var context = new UniversityDatabase();
            var activity = context.Activities.FirstOrDefault(x => x.Id == model.Id);
            if (activity == null)
            {
                return null;
            }
            activity.Update(model);
            context.SaveChanges();
            return activity.GetViewModel;
        }
        public ActivityViewModel? Delete(ActivityBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.Activities.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Activities.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
