﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;

namespace UniversityDatabaseImplement.Implements
{
    internal class ReportTypeStorage : IReportTypeStorage
    {
        public List<ReportTypeViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.ReportTypes.Select(x => x.GetViewModel).ToList();
        }
        public List<ReportTypeViewModel> GetFilteredList(ReportTypeSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.ReportTypes
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id) &&
                        (string.IsNullOrEmpty(model.Name) || x.Name.Contains(model.Name))
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ReportTypeViewModel? GetElement(ReportTypeSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.ReportTypes
            .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.Name == model.Name) ||
                                 (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public ReportTypeViewModel? Insert(ReportTypeBindingModel model)
        {
            var newReportType = ReportType.Create(model);
            if (newReportType == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.ReportTypes.Add(newReportType);
            context.SaveChanges();
            return newReportType.GetViewModel;
        }

        public ReportTypeViewModel? Update(ReportTypeBindingModel model)
        {
            using var context = new UniversityDatabase();
            var reportType = context.ReportTypes.FirstOrDefault(x => x.Id == model.Id);
            if (reportType == null)
            {
                return null;
            }
            reportType.Update(model);
            context.SaveChanges();
            return reportType.GetViewModel;
        }
        public ReportTypeViewModel? Delete(ReportTypeBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.ReportTypes.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.ReportTypes.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
